"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = __importStar(require("lodash"));
var defaultConfig = {
    startOfStringBonus: 1,
    startOfWordBonus: 1,
    allowErrors: 0,
    matchProportionBonus: 0,
    markBefore: "*",
    markAfter: "*",
    escape: true,
    deburr: true
};
function escapeRegExp(text) {
    return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, "\\$&");
}
var Search = /** @class */ (function () {
    function Search(config) {
        this.searchTerm = "";
        this.sensitive = false;
        this.config = _.assign({}, defaultConfig, config);
    }
    Search.prototype.term = function (searchTerm) {
        this.sensitive = /[^a-z .@\-]/.test(searchTerm);
        this.searchTerm = this.config.escape ? escapeRegExp(searchTerm) : searchTerm;
        return this;
    };
    Search.prototype.in = function (str) {
        var sensitive = this.sensitive;
        var term = this.searchTerm;
        var re = function (pattern) { return new RegExp(pattern, sensitive ? "" : "i"); };
        var deburrStr = this.config.deburr ? _.deburr(str) : str;
        var match;
        var score = 3;
        var patterns = ["^" + term, "\\b" + term, term];
        for (var _i = 0, patterns_1 = patterns; _i < patterns_1.length; _i++) {
            var pattern = patterns_1[_i];
            match = re(pattern).exec(deburrStr);
            if (match) {
                var spans = [[match.index, match.index + match[0].length]];
                var marked = this.mark(str, spans);
                return { score: score, spans: spans, marked: marked };
            }
            else {
                score--;
            }
        }
        return { score: score, spans: [], marked: str };
    };
    Search.prototype.mark = function (str, spans) {
        var b = this.config.markBefore;
        var a = this.config.markAfter;
        var ret = "";
        var p = 0;
        var normalized = _.map(spans, function (span) { return (_.isNumber(span) ? [span, span + 1] : span); });
        _.each(normalized, function (span) {
            ret += "" + str.substring(p, span[0]) + b + str.substring(span[0], span[1]) + a;
            p = span[1];
        });
        ret += str.substring(p);
        return ret;
    };
    return Search;
}());
exports.Search = Search;
